/*
 *
 *
 *    codec.h
 *
 *    SGTL5000 codec configuration.
 *
 *    License: GNU General Public License Version 3.0.
 *
 *    Copyright (C) 2019 by Matthew K. Roberts, KK5JY. All rights reserved.
 *
 *    This program is free software: you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation, either version 3 of the License, or
 *    (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful, but
 *    WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *    General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program.  If not, see: http://www.gnu.org/licenses/
 *
 *
 */

#ifndef __ANYRIG_CODEC_H
#define __ANYRIG_CODEC_H

// ------------------------- BEGIN CONFIGURATION ------------------------- 

//
//  Several constant values can be configured below.  The defaults
//  provided are reasonble for most purposes.
//

//
//  Select the input source, from either:
//
//    AUDIO_INPUT_MIC
//    AUDIO_INPUT_LINEIN
//
//  The LINE IN ports are compatible with most modern radio
//  receivers, and the LINE IN gain set by adjusting the value
//  of ANYRIG_LINE_IN_GAIN, below.
//
//  If extra sensitivity is required, the microphone input can
//  be used instead, and the mic gain configured by setting
//  the value of ANYRIG_MIC_GAIN, below.
//
#define ANYRIG_INPUT (AUDIO_INPUT_LINEIN) // or AUDIO_INPUT_MIC

//
//  Select the MIC input gain, in decibels; valid range is 0 to 63
//
//  This setting only applies when using AUDIO_INPUT_MIC as the
//  audio source.
//
#define ANYRIG_MIC_GAIN (63) // dB, 0 to 63

//
//  Select the LINE input gain, from one of these values:
//
//      0: 3.12 Volts p-p
//      1: 2.63 Volts p-p
//      2: 2.22 Volts p-p
//      3: 1.87 Volts p-p
//      4: 1.58 Volts p-p
//      5: 1.33 Volts p-p  (default)
//      6: 1.11 Volts p-p
//      7: 0.94 Volts p-p
//      8: 0.79 Volts p-p
//      9: 0.67 Volts p-p
//     10: 0.56 Volts p-p
//     11: 0.48 Volts p-p
//     12: 0.40 Volts p-p
//     13: 0.34 Volts p-p
//     14: 0.29 Volts p-p
//     15: 0.24 Volts p-p
//
#define ANYRIG_LINE_IN_GAIN (5)

// -------------------------  END  CONFIGURATION ------------------------- 


//
//  configure_codec(...)
//
inline void configure_codec(AudioControlSGTL5000 *codec) {
	// Start the SGTL5000. This function should be called first.
	codec->enable();

	// mute everything before configuring
	codec->muteHeadphone();
	codec->muteLineout();

	// Select which input to use: AUDIO_INPUT_LINEIN or AUDIO_INPUT_MIC.
	codec->inputSelect(ANYRIG_INPUT);

	// When using the microphone input, set the amplifier gain. The input number is
	// in decibels, from 0 to 63.
	codec->micGain(ANYRIG_MIC_GAIN);

	// set the LINE IN gain
	codec->lineInLevel(ANYRIG_LINE_IN_GAIN);

	// set the D/A conversion volume; this should be left at 1.0, to provide maximum resolution
	codec->dacVolume(1.0);

	// disable surround processing
	codec->surroundSoundDisable();

	// select FLAT equalizer
	codec->eqSelect(FLAT_FREQUENCY);

	// unmute the outputs
	codec->unmuteHeadphone();
	codec->unmuteLineout();
}


//
//  set_codec_volume(...)
//
//       level = normalized volume, 0.0 to 1.0
//
inline void set_codec_volume(AudioControlSGTL5000 *codec, float level) {
	// set the headphone volume
	codec->volume(level);

	// Set the line-out level; this isn't volume per se, but it will still
	//   cause the line output level to follow the software mixer;
	//   the LINE IN level control has roughly 8.7dB between its highest
	//   and lowest values...
	//
	// valid values:
	//   13: 3.16 Volts p-p
	//   14: 2.98 Volts p-p
	//   15: 2.83 Volts p-p
	//   16: 2.67 Volts p-p
	//   17: 2.53 Volts p-p
	//   18: 2.39 Volts p-p
	//   19: 2.26 Volts p-p
	//   20: 2.14 Volts p-p
	//   21: 2.02 Volts p-p
	//   22: 1.91 Volts p-p
	//   23: 1.80 Volts p-p
	//   24: 1.71 Volts p-p
	//   25: 1.62 Volts p-p
	//   26: 1.53 Volts p-p
	//   27: 1.44 Volts p-p
	//   28: 1.37 Volts p-p
	//   29: 1.29 Volts p-p  (default)
	//   30: 1.22 Volts p-p
	//   31: 1.16 Volts p-p
	//

	// normalize the value of 'level' to the range of usable values
	int newLevel = 31 - ((31 - 13) * level);
	codec->lineOutLevel(newLevel);
}

#endif
