/*
 *
 *
 *    modules.h
 *
 *    Configuration for output modules.
 *
 *    License: GNU General Public License Version 3.0.
 *    
 *    Copyright (C) 2019 by Matthew K. Roberts, KK5JY. All rights reserved.
 *
 *    This program is free software: you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation, either version 3 of the License, or
 *    (at your option) any later version.
 *    
 *    This program is distributed in the hope that it will be useful, but
 *    WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *    General Public License for more details.
 *    
 *    You should have received a copy of the GNU General Public License
 *    along with this program.  If not, see: http://www.gnu.org/licenses/
 *
 *
 */

#ifndef ANYRIG_MODULES_H
#define ANYRIG_MODULES_H

//
//  This is the configuration for the output modules.
//
//  There are currently two different module types supported:
//
//     1. The 'VOX' module.  This module drives a PTT output through a
//        transistor, with a configurable hang-time.  The module also
//        supports an optional potentiometer to adjust the hang-time.
//     2. The 'Key' module.  This module drives a CW key or RTTY FSK
//        transmitter through a transistor, with constant rising- and
//        trailing-edge timing.
//
//  You can configure up to one module per channel, using the two
//  functions below.  Either function can return 'null', which will
//  assign no module to that channel.
//
//  Audio for either channel will continue to flow, regardless of
//  whether an output module is assigned to that channel.
//

//
//  This is the LEFT module configuration
//
OutputModule *ConfigureLeftModule() {
	//
	//  Create a new VOX module
	//
	VOX *result = new VOX();

	//
	//  Select the PTT pin number; this is a DIGITAL output that
	//    drives a BJT, FET, or RELAY to enable the transmitter.
	//
	//  If you need extra current to drive two output LEDs, such
	//    as a combination of optoisolator and visible LED, it is
	//    recommended that the 'LedPin' be used for one, and the
	//    'OutPin' used for the other, to keep per-pin output
	//    current within the specifications of the Teensy device.
	//
	result->OutPin(2); // D2

	//
	//  Select the LED pin number; this is a DIGITAL output that
	//    drives an optional LED, to show the output state of the
	//    module.
	//
	//  Since the per-pin current limits of the Teensy are rather
	//    low, this provides a dedicated output for an LED that
	//    reflects the output state.  This is mostly useful when
	//    trying to drive both an optoisolator LED and a visual
	//    LED from the same module.
	//
	//  Set to (-1) to disable the LED output.
	//
	result->LedPin(3); // D3

	//
	//  Select the VOX DELAY potentiometer input; this is an A/D
	//    input used to control the VOX delay value, instead of
	//    using a fixed value.
	//
	//  Set this to (-1) to disable the potentiometer.
	//
	result->PotPin(A7); // A7 (same pin as D21)

	//
	//  Select the trigger methods.
	//
	//  Set UseAudio to 'true' to enable keying based on the audio
	//    level exceeding its threshold.
	//  Set UseRTS to 'true' to enable keying based on the Teensy's
	//    USB port RTS line state.
	//  Set UseDTR to 'true' to enable keying based on the Teensy's
	//    USB port DTR line state.
	//
	result->UseAudio(true);
	result->UseRTS(false);
	result->UseDTR(false);

	//
	//  Set the initial VOX DELAY, measured in milliseconds.
	//
	//  This is the "tail" or "hang" time that the VOX circuit stays
	//     in transmit after the most recent transmit audio peak above
	//     the threshold.
	//
	//  If a potentiometer is configured above, this value will be modified
	//    by reading the potentiometer value.
	//
	result->VoxDelay(300); // msec

	// return the new module
	return result;
}


//
//  This is the RIGHT module configuration
//
OutputModule *ConfigureRightModule() {
	//
	//  Create a new KEY module
	//
	Key *result = new Key();

	//
	//  Select the KEY pin number; this is a DIGITAL output that
	//    drives a BJT or FET for keying the key input of a CW
	//    transmitter.
	//
	//  If you need extra current to drive two output LEDs, such
	//    as a combination of optoisolator and visible LED, it is
	//    recommended that the 'LedPin' be used for one, and the
	//    'OutPin' used for the other, to keep per-pin output
	//    current within the specifications of the Teensy device.
	//
	result->OutPin(4); // D5

	//
	//  Select the LED pin number; this is a DIGITAL output that
	//    drives an optional LED, to show the output state of the
	//    module.
	//
	//  Since the per-pin current limits of the Teensy are rather
	//    low, this provides a dedicated output for an LED that
	//    reflects the output state.  This is mostly useful when
	//    trying to drive both an optoisolator LED and a visual
	//    LED from the same module.
	//
	//  Set to (-1) to disable the LED output.
	//
	result->LedPin(5); // D5

	//
	//  Select the trigger methods.
	//
	//  Set UseAudio to 'true' to enable keying based on the audio
	//    level exceeding its threshold.
	//  Set UseRTS to 'true' to enable keying based on the Teensy's
	//    USB port RTS line state.
	//  Set UseDTR to 'true' to enable keying based on the Teensy's
	//    USB port DTR line state.
	//
	result->UseAudio(true);
	result->UseRTS(false);
	result->UseDTR(false);

	//
	//  Select the KEY DELAY value; 
	//
	//  This is similar to VOX delay, but it uses a much shorter
	//     time delay, since CW and FSK keying need more responsive
	//     transition timing, and the delay needs to be applied
	//     to both the rising and falling edges of the envelope,
	//     to accurately reproduce the keying envelope duration.
	//
	//  The default value of 10ms should be sufficient for CW speeds
	//     up to around 120WPM, and RTTY speeds up to around 100bps.
	//
	result->KeyDelay(10); // msec

	// return the new module
	return result;
}

#endif
