/*
 *
 *
 *    vox.h
 *
 *    VOX output module.
 *
 *    License: GNU General Public License Version 3.0.
 *    
 *    Copyright (C) 2019 by Matthew K. Roberts, KK5JY. All rights reserved.
 *
 *    This program is free software: you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation, either version 3 of the License, or
 *    (at your option) any later version.
 *    
 *    This program is distributed in the hope that it will be useful, but
 *    WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *    General Public License for more details.
 *    
 *    You should have received a copy of the GNU General Public License
 *    along with this program.  If not, see: http://www.gnu.org/licenses/
 *
 *
 */

#ifndef __KK5JY_VOX_H
#define __KK5JY_VOX_H

// ------------------------- BEGIN CONFIGURATION ------------------------- 

//
//  Several constant values can be configured below.  The defaults
//  provided are reasonble for most purposes.
//

//
//  Set the default VOX delay, measured in milliseconds.
//
//  This is the "tail" or "hang" time that the VOX circuit stays
//     in transmit after the most recent transmit audio peak above
//     the threshold.
//
//  This can be overridden by an optional potentiometer, if that
//    feature is enabled.
//
#define DEFAULT_VOX_DELAY (300) // msec

//
//  Set the minimum VOX delay/hang/tail (msec) that can be selected via
//    the potentiometer
//
#define MIN_VOX_DELAY (50)   // msec

//
//  Set the maximum VOX delay/hang/tail (msec) that can be selected via
//    the potentiometer
//
#define MAX_VOX_DELAY (1500) // msec

//
//  Set how often to process the potentiometer (msec)
//
#define POT_INTERVAL (50)   // msec

//
//  Define PTT output states
//
#define PTT_HIGH (HIGH)
#define PTT_LOW (LOW)

//
//  Define the normalized level at which the clip warning LED should light
//
#define CLIP_LIMIT (0.6725) // 1dB below clipping, RMS

// -------------------------  END  CONFIGURATION ------------------------- 

// include interfaces
#include "output.h"
#include "Elapsed.h"


//
//  class VOX
//
class VOX : public OutputModule {
	private: // config
		// delay in msec
		uint16_t c_delay;
		// potentiometer pin
		int16_t c_potPin;
		// output pin
		int16_t c_outPin;
		// LED pin
		int16_t c_ledPin;
		// Audio flag
		bool c_useAudio;
		// DTR flag
		bool c_useDtr;
		// RTS flag
		bool c_useRts;
	
	public:
		int16_t VoxDelay() const { return c_delay; }
		void    VoxDelay(int16_t d) { c_delay = d; }

		int16_t PotPin() const { return c_potPin; }
		void    PotPin(int16_t d) { c_potPin = d; }

		int16_t OutPin() const { return c_outPin; }
		void    OutPin(int16_t d) { c_outPin = d; }

		int16_t LedPin() const { return c_ledPin; }
		void    LedPin(int16_t d) { c_ledPin = d; }

		bool UseAudio() const { return c_useAudio; }
		void UseAudio(bool b) { c_useAudio = b; }

		bool UseDTR() const { return c_useDtr; }
		void UseDTR(bool b) { c_useDtr = b; }

		bool UseRTS() const { return c_useRts; }
		void UseRTS(bool b) { c_useRts = b; }

	private: // state
		// the TX flag, which matches the VOX input
		bool m_tx;

		// the PTT flag, which matches the gate output
		bool m_ptt;

		// the last time the VOX audio was above the threshold
		uint32_t m_lastVOX;

		// last potentiometer read
		uint32_t m_lastPot;

		// the VOX timeout/delay, measured in msec
		uint16_t m_voxDelay;

		// the VOX timeout/delay raw value
		int16_t m_voxInput;

	public:
		//
		//  constructor
		//
		VOX() {
			c_delay = DEFAULT_VOX_DELAY;
			c_potPin = -1;
			c_outPin = -1;
			c_ledPin = -1;

			c_useAudio = true;
			c_useRts = false;
			c_useDtr = false;

			m_tx = false;
			m_ptt = false;
			m_lastVOX = 0;
			m_lastPot = 0;
			m_voxDelay = 0;
			m_voxInput = -100; // force initial value read
		}
	
	public:
		//
		//  Setup()
		//
		void Setup() {
			if (c_potPin >= 0) {
				// initialize DELAY_POT pin
				pinMode(c_potPin, INPUT);
			}

			if (c_outPin >= 0) {
				// initialize PTT pin
				pinMode(c_outPin, OUTPUT);
				digitalWrite(c_outPin, PTT_LOW);
			}

			if (c_ledPin >= 0) {
				// initialize LED pin
				pinMode(c_ledPin, OUTPUT);
				digitalWrite(c_ledPin, LOW);
			}
		}


		//
		//  Loop()
		//
		void Loop() {
			// read the clock
			uint32_t now = millis();

			// update VOX delay based on potentiometer
			if (c_potPin >= 0) {
				if (Elapsed(now, m_lastPot) >= POT_INTERVAL) {
					m_lastPot = now;
					int16_t newValue = analogRead(c_potPin);
					if (abs(newValue - m_voxInput) >= 3) {
						// compute the current delay based on pot position
						m_voxInput = newValue;
						int16_t i = MIN_VOX_DELAY +
							(static_cast<int16_t>(MAX_VOX_DELAY - MIN_VOX_DELAY) *
								static_cast<int16_t>(m_voxInput)) / 1020;

						// round to nearest 10ms
						i /= 10;
						i *= 10;

						// clamp if there was rounding/truncation error
						if (i < MIN_VOX_DELAY)
							i = MIN_VOX_DELAY;
						if (i > MAX_VOX_DELAY)
							i = MAX_VOX_DELAY;

						// store new value
						m_voxDelay = i;

						#ifdef VERBOSE_DEBUG
						Serial.print("New VOX delay: ");
						Serial.print(m_voxDelay);
						Serial.println(" ms");
						#endif
					}
				}
			}

			// process VOX timeout
			if (m_ptt && (Elapsed(millis(), m_lastVOX) >= m_voxDelay)) {
				if (c_outPin >= 0) {
					digitalWrite(c_outPin, PTT_LOW);
				}
				if (c_ledPin >= 0) {
					digitalWrite(c_ledPin, LOW);
				}
				m_ptt = false;

				#ifdef VERBOSE_DEBUG
				Serial.print("New PTT state: ");
				Serial.print(m_ptt);
				Serial.print("; delay = ");
				Serial.println(m_voxDelay);
				#endif
			}
		}


		//
		//  SetState(...)
		//
		void SetState(bool new_tx) {
			// refresh the VOX timer
			if (new_tx) {
				m_lastVOX = millis();
			}

			// if the state has changed...
			if (m_tx != new_tx) {
				m_tx = new_tx;

				if (m_tx) {
					m_ptt = true;

					if (c_outPin >= 0) {
						// assert PTT
						digitalWrite(c_outPin, PTT_HIGH);
					}

					if (c_ledPin >= 0) {
						// assert LED
						digitalWrite(c_ledPin, HIGH);
					}

					#ifdef VERBOSE_DEBUG
					Serial.print("New PTT state: ");
					Serial.println(m_ptt);
					#endif
				}
			}
		}


		//
		//  ClipWarning() - return the level at which the clipping LED should light
		//
		float ClipWarning() {
			return CLIP_LIMIT;
		}
};

#endif // __KK5JY_VOX_H
